//! # Running Code on Cleanup with the Drop Trait
//!
//! Second trait important to the smart pointer is `Drop`, lets
//! customize what happens when value is about to go out of scope.

use std::mem::drop;

struct CustomSmartPointer {
    data: String,
}

impl Drop for CustomSmartPointer {
    fn drop(&mut self) {
        println!("Dropping CustomSmartPointer with data `{}`!", self.data);
    }
}

fn main() {
    let c = CustomSmartPointer {
        data: String::from("my stuff"),
    };
    let d = CustomSmartPointer {
        data: String::from("other stuff"),
    };
    println!("CustomSmartPointers created.");

    let c = CustomSmartPointer {
        data: String::from("some data"),
    };
    println!("CustomSmartPointer created.");
    // c.drop(); // not allowed to explictly call drop like this.
    drop(c);
    println!("CustomSmartPointer dropped before the end of main.");
}
