use std::thread;
use std::time::Duration;

fn main() {
    let handle = thread::spawn(|| {
        for i in 1..10 {
            println!("hi number {i} from the spawned thread!");
            thread::sleep(Duration::from_millis(1));
        }
    });

    for i in 1..5 {
        println!("hi number {i} from the main thread!");
        thread::sleep(Duration::from_millis(1));
    }

    // Waiting for all threads to finish using join handles
    // Without the join below, threads stop prematurely
    // because there is no guarantee on the order in which
    // threads run, also can't guarantee spawned thread
    // will get to run at all.

    // calling `join` on a handle blocks the thread currently
    // running until the thread represented by `handle`
    // terminates.
    handle.join().unwrap();

    // Using move closures with threads
    //
    // Often use move keyword with closures because
    // closure takes ownership fo values it uses from environment.

    let v = vec![1, 2, 3];
    let handle = thread::spawn(move || {
        println!("Here's a vector: {v:?}");
    });
    handle.join().unwrap();
}
